/***************************************************************************

TITLE:		F3F.c

----------------------------------------------------------------------------

FUNCTION:	routine for F3f race handling

----------------------------------------------------------------------------

MODULE STATUS:	Developmental

----------------------------------------------------------------------------

GENEALOGY:	Written 08 december 01

----------------------------------------------------------------------------

DESIGNED BY:	OBR

CODED BY:	 OBR

MAINTAINED BY:	OBR

----------------------------------------------------------------------------*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "inc\gl\glut.h"
#ifdef WIN32
//#include <MAPIWIN.H>
#endif

#include "ls_types.h"
#include "ls_constants.h"
#include "ls_generic.h"

#include "f3f.h"

extern char sound_penalty[];
extern char sound_file[];
extern int repeat_mode;
extern int security_line;


char f3f_soundfile [256];
char f3f_soundpenalty [256]; 

unsigned char f3f_score [1024] = "";
unsigned char f3f_time [256] = "";
unsigned char f3f_turntime[256] = "";
unsigned char f3f_meters [256] = "";
unsigned char f3f_result [512] = "";
unsigned char f3f_penalty [256] = "";

int pausetime = 0;
int currtime = 0;
int runstarttime = 0;
int runtime = 0;
int runlaststart = 0;
int runcurrentstart=0;
int runcurrentstop=0;
int runcurrentturn=0;
int runturntime = 0;

int run_will_start = FALSE;
int run_started = FALSE;
int run_completed = FALSE;
int start_on_left = TRUE;
int x1_toggle = FALSE;
int x2_toggle = FALSE;

int base_count =-1;
int next_base_to_cross = PYLON_RIGHT;
int in_penalty = TRUE;
int penality_count=0;
int start_from_ground = TRUE;

int elapsed_time[MAX_LAPS+1];
int lostm[MAX_LAPS+1];
int turntime[MAX_LAPS+1];

int lost_meters_rel = 0;
int lost_meters_abs = 0;

void PlayStart ()
{
	char sfile [256];
	if (strcmp(sound_file,"")==0) printf ("\007");
	else {
		sprintf (sfile,"%s%d.wav",sound_file,-2);
		PlaySound( sfile, NULL, SND_ASYNC |SND_FILENAME);
	}
}

void PlayEntry ()
{
	char sfile [256];
  if (strcmp(sound_file,"")==0) printf ("\007");
	else {
		sprintf (sfile,"%s%d.wav",sound_file,-1);
		PlaySound( sfile, NULL, SND_ASYNC |SND_FILENAME|SND_NOSTOP  );
	}
}

void PlayFirst ()
{
	char sfile [256];
	if (strcmp(sound_file,"")==0) printf ("\007");
	else {
		sprintf (sfile,"%s%d.wav",sound_file,0);
		PlaySound( sfile, NULL, SND_ASYNC |SND_FILENAME);
	}
}

void Playbase ()
{
	char sfile [256];
	if (strcmp(sound_file,"")==0) printf ("\007");
	else {
		sprintf (sfile,"%s%d.wav",sound_file,base_count+1);
		PlaySound( sfile, NULL, SND_ASYNC |SND_FILENAME);
	}
}

void Playpenalty ()
{
	if (strcmp(sound_penalty,"")==0) printf ("\007");
	else PlaySound (sound_penalty, NULL, SND_FILENAME |SND_ASYNC);
}


void update_time ()
{
	currtime = glutGet(GLUT_ELAPSED_TIME);
	runtime = currtime - runstarttime - pausetime;
}


void restart_f3f()
{
	run_will_start = FALSE;
	run_started = FALSE;
	run_completed = FALSE;
	x1_toggle = FALSE;
	x2_toggle = FALSE;
	if (start_on_left == TRUE)
		next_base_to_cross = PYLON_RIGHT;
	else
		next_base_to_cross = PYLON_LEFT;
	base_count = 0;
	penality_count = 0;
	in_penalty = TRUE;

	pausetime = 0;
	currtime = 0;
	runstarttime = 0;
	runtime = 0;
	runcurrentstart = 0;
	runlaststart = 0;
	runcurrentstop = 0;
	runcurrentturn = 0;
	runturntime = 0;

	lost_meters_rel = 0;
	lost_meters_abs = 0;

	f3f_score [0] = '\0';
	f3f_meters [0] = '\0';
	f3f_turntime [0] = '\0';
	f3f_result [0] = '\0';
	f3f_time [0] = '\0';
	f3f_penalty [0] = '\0';
	UPDATE_START;
  PlayStart();
}


/*
Purpose: Get Time when starting the run
*/
void run_start()
{
	update_time ();
	UPDATE_START;

	if (start_on_left == TRUE) /* start  on left */ {

		if ((X_cg_rwy > PLANLIMIT) && (run_will_start == FALSE))  {
			run_will_start = TRUE;
			PlayEntry ();
		}
		if (((X_cg_rwy < PLANLIMIT) && (run_will_start == TRUE)) || repeat_mode == 2) {
			run_started = TRUE;
			PlayFirst ();
			update_time ();
			if (runtime > MAX_START_TIME) runstarttime = MAX_START_TIME;
			else runstarttime = runtime;
		}
	}
	else  /* start on right */ {
		if ((X_cg_rwy < -PLANLIMIT) && (run_will_start == FALSE)) {
			run_will_start = TRUE;
			Playbase ();
		}
		if (((X_cg_rwy > -PLANLIMIT) && (run_will_start == TRUE)) || repeat_mode == 2) {
			run_started = TRUE;
			Playbase ();
			update_time ();
			if (runtime > MAX_START_TIME) runstarttime = MAX_START_TIME;
			else runstarttime = runtime;
		}
	}
}


void end_run()
{
	run_completed = TRUE;

	/* save result in score file */
	{
		FILE *fp;
		char thetime [256];
		char thedate [256];
		char thestart [256];

		fp = fopen ("f3f_score.txt","a+");
		_strtime(thetime);
		_strdate(thedate);
		if (start_from_ground == TRUE)
			sprintf (thestart,"Start from ground");
		else
			sprintf (thestart,"Start from altitude");

		fprintf (fp,"%s, %s, Run: %2d.%2ds, Penalty: %d, Lost: %dm, Turn: %2d.%2ds, %s\n", thedate, thetime,
				(runtime)/1000, (runtime)%1000/10, penality_count, lost_meters_abs, (runturntime)/1000, (runturntime)%1000/10, thestart);
		fclose (fp);
	}
}


/*
	 calculate the meters run over the base in the turn 
	 */
void update_lost_meters ()
{
	double x = X_cg_rwy;

	if (x < 0) x = -x;

	x = (x - PLANLIMIT)*0.3048;

	if (x > lost_meters_rel)
	{
		// actual glider pos - pylon pos
		lost_meters_rel = (int) x;
	}
}


void update_turntime ()
{
	if ((X_cg_rwy > PLANLIMIT) && (next_base_to_cross == PYLON_RIGHT))
	{
		update_time ();
		runcurrentturn = runtime;
	}
	if ((X_cg_rwy < -PLANLIMIT) && (next_base_to_cross == PYLON_LEFT))
	{
		update_time ();
		runcurrentturn = runtime;
	}
}


/*
	 check if the model cross pylons A or pylons B
	 Increase BASE counter
	 */
void f3f_main_loop ()
{
	// start countdown
	if (run_started == FALSE && run_completed == FALSE)
	{
		run_start();
		if (run_started == FALSE) return;
	}

	// penalties verhaengen
	if (Y_cg_rwy < security_line/0.3048) {
		if (in_penalty == FALSE) {
			if (run_completed == FALSE) {
				penality_count++;
			}
			in_penalty = TRUE;
			if (run_completed == FALSE) Playpenalty ();
		}
	}
	else in_penalty = FALSE;


	// check if the model cross pylons A or pylons B
	if ((X_cg_rwy > PLANLIMIT) && (next_base_to_cross == PYLON_LEFT)) {
		next_base_to_cross = PYLON_RIGHT;
		Playbase ();

		if (run_completed == FALSE) {
			base_count++;
			turntime[base_count-1] = runcurrentturn - runcurrentstop;
			runturntime += (runcurrentturn - runcurrentstop);
			update_time ();
			if (base_count > 0) {
				runcurrentstop = runtime;
			}
			elapsed_time[base_count] = runcurrentstop - runcurrentstart;
			runlaststart = runcurrentstart;
			runcurrentstart = runtime;
			lostm[base_count-1] = lost_meters_rel;
			lost_meters_abs += lost_meters_rel;
			lost_meters_rel = 0;
			if (base_count >= MAX_LAPS) {
				end_run();
			}
		}
	}
	else {
		if ((X_cg_rwy < -PLANLIMIT) && (next_base_to_cross == PYLON_RIGHT)) {
			next_base_to_cross = PYLON_LEFT;
			Playbase ();

			if (run_completed == FALSE) {
				base_count++;
				turntime[base_count-1] = runcurrentturn - runcurrentstop;
				runturntime += (runcurrentturn - runcurrentstop);
				update_time ();
				if (base_count > 0) {
					runcurrentstop = runtime;
				}
				elapsed_time[base_count] = runcurrentstop - runcurrentstart;
				runlaststart = runcurrentstart;
				runcurrentstart = runtime;
				lostm[base_count-1] = lost_meters_rel;
				lost_meters_abs += lost_meters_rel;
				lost_meters_rel = 0;
				if (base_count == MAX_LAPS) {
					end_run();
				}
			}
		}
	}

	// status update
	if (run_completed == TRUE) {
		UPDATE_COMPLETE;
	}
	else if (run_started == TRUE) {
		UPDATE_SCORE;
		UPDATE_PENALTY;
		update_time ();
		UPDATE_TIME;
		update_lost_meters ();
		UPDATE_METERS;
		update_turntime ();
		UPDATE_TURN;
	}
}


void beep(){
	char sound_file [256];
	sprintf (sound_file,"%s%d.wav",f3f_soundfile,base_count+1);
	PlaySound( sound_file, NULL, SND_ASYNC |SND_FILENAME);
}
void beep2(){
	PlaySound( f3f_soundpenalty, NULL, SND_ASYNC |SND_FILENAME);
}
